@@ -23,7 +23,8 @@ from statsmodels.tsa.stattools import (adfuller, acf, pacf_yw, pacf_ols,
                                        coint, acovf, kpss,
                                        arma_order_select_ic, levinson_durbin,
                                        levinson_durbin_pacf, pacf_burg,
-                                       innovations_algo, innovations_filter)
+                                       innovations_algo, innovations_filter,
+                                       zivot_andrews)
 
 DECIMAL_8 = 8
 DECIMAL_6 = 6
@@ -33,8 +34,10 @@ DECIMAL_3 = 3
 DECIMAL_2 = 2
 DECIMAL_1 = 1
 
+CURR_DIR = os.path.dirname(os.path.abspath(__file__))
 
-@pytest.fixture('module')
+
+@pytest.fixture(scope='module')
 def acovf_data():
     rnd = np.random.RandomState(12345)
     return rnd.randn(250)
@@ -66,10 +69,11 @@ class TestADFConstant(CheckADF):
     """
     Dickey-Fuller test for unit root
     """
+
     @classmethod
     def setup_class(cls):
         cls.res1 = adfuller(cls.x, regression="c", autolag=None,
-                maxlag=4)
+                            maxlag=4)
         cls.teststat = .97505319
         cls.pvalue = .99399563
         cls.critvalues = [-3.476, -2.883, -2.573]
@@ -78,29 +82,32 @@ class TestADFConstant(CheckADF):
 class TestADFConstantTrend(CheckADF):
     """
     """
+
     @classmethod
     def setup_class(cls):
         cls.res1 = adfuller(cls.x, regression="ct", autolag=None,
-                maxlag=4)
+                            maxlag=4)
         cls.teststat = -1.8566374
         cls.pvalue = .67682968
         cls.critvalues = [-4.007, -3.437, -3.137]
 
 
-#class TestADFConstantTrendSquared(CheckADF):
+# FIXME: do not leave commented-out
+# class TestADFConstantTrendSquared(CheckADF):
 #    """
 #    """
 #    pass
-#TODO: get test values from R?
+# TODO: get test values from R?
 
 
 class TestADFNoConstant(CheckADF):
     """
     """
+
     @classmethod
     def setup_class(cls):
         cls.res1 = adfuller(cls.x, regression="nc", autolag=None,
-                maxlag=4)
+                            maxlag=4)
         cls.teststat = 3.5227498
 
         cls.pvalue = .99999
@@ -117,7 +124,7 @@ class TestADFConstant2(CheckADF):
     @classmethod
     def setup_class(cls):
         cls.res1 = adfuller(cls.y, regression="c", autolag=None,
-                maxlag=1)
+                            maxlag=1)
         cls.teststat = -4.3346988
         cls.pvalue = .00038661
         cls.critvalues = [-3.476, -2.883, -2.573]
@@ -127,7 +134,7 @@ class TestADFConstantTrend2(CheckADF):
     @classmethod
     def setup_class(cls):
         cls.res1 = adfuller(cls.y, regression="ct", autolag=None,
-                maxlag=1)
+                            maxlag=1)
         cls.teststat = -4.425093
         cls.pvalue = .00199633
         cls.critvalues = [-4.006, -3.437, -3.137]
@@ -137,14 +144,14 @@ class TestADFNoConstant2(CheckADF):
     @classmethod
     def setup_class(cls):
         cls.res1 = adfuller(cls.y, regression="nc", autolag=None,
-                maxlag=1)
+                            maxlag=1)
         cls.teststat = -2.4511596
         cls.pvalue = 0.013747
         # Stata does not return a p-value for noconstant
         # this value is just taken from our results
-        cls.critvalues = [-2.587,-1.950,-1.617]
+        cls.critvalues = [-2.587, -1.950, -1.617]
         _, _1, _2, cls.store = adfuller(cls.y, regression="nc", autolag=None,
-                                         maxlag=1, store=True)
+                                        maxlag=1, store=True)
 
     def test_store_str(self):
         assert_equal(self.store.__str__(), 'Augmented Dickey-Fuller Test Results')
@@ -156,40 +163,38 @@ class CheckCorrGram(object):
     """
     data = macrodata.load_pandas()
     x = data.data['realgdp']
-    filename = os.path.dirname(os.path.abspath(__file__))+\
-            "/results/results_corrgram.csv"
+    filename = os.path.join(CURR_DIR, 'results', 'results_corrgram.csv')
     results = pd.read_csv(filename, delimiter=',')
 
-    #not needed: add 1. for lag zero
-    #self.results['acvar'] = np.concatenate(([1.], self.results['acvar']))
-
 
 class TestACF(CheckCorrGram):
     """
     Test Autocorrelation Function
     """
+
     @classmethod
     def setup_class(cls):
         cls.acf = cls.results['acvar']
-        #cls.acf = np.concatenate(([1.], cls.acf))
+        # cls.acf = np.concatenate(([1.], cls.acf))
         cls.qstat = cls.results['Q1']
         cls.res1 = acf(cls.x, nlags=40, qstat=True, alpha=.05, fft=False)
-        cls.confint_res = cls.results[['acvar_lb','acvar_ub']].values
+        cls.confint_res = cls.results[['acvar_lb', 'acvar_ub']].values
 
     def test_acf(self):
         assert_almost_equal(self.res1[0][1:41], self.acf, DECIMAL_8)
 
     def test_confint(self):
-        centered = self.res1[1] - self.res1[1].mean(1)[:,None]
+        centered = self.res1[1] - self.res1[1].mean(1)[:, None]
         assert_almost_equal(centered[1:41], self.confint_res, DECIMAL_8)
 
     def test_qstat(self):
         assert_almost_equal(self.res1[2][:40], self.qstat, DECIMAL_3)
         # 3 decimal places because of stata rounding
 
-#    def pvalue(self):
-#        pass
-#NOTE: shouldn't need testing if Q stat is correct
+    # FIXME: enable/xfail/skip or delete
+    # def pvalue(self):
+    #    pass
+    # NOTE: should not need testing if Q stat is correct
 
 
 class TestACF_FFT(CheckCorrGram):
@@ -204,7 +209,7 @@ class TestACF_FFT(CheckCorrGram):
         assert_almost_equal(self.res1[0][1:], self.acf, DECIMAL_8)
 
     def test_qstat(self):
-        #todo why is res1/qstat 1 short
+        # todo why is res1/qstat 1 short
         assert_almost_equal(self.res1[1], self.qstat, DECIMAL_3)
 
 
@@ -212,8 +217,8 @@ class TestACFMissing(CheckCorrGram):
     # Test Autocorrelation Function using Missing
     @classmethod
     def setup_class(cls):
-        cls.x = np.concatenate((np.array([np.nan]),cls.x))
-        cls.acf = cls.results['acvar'] # drop and conservative
+        cls.x = np.concatenate((np.array([np.nan]), cls.x))
+        cls.acf = cls.results['acvar']  # drop and conservative
         cls.qstat = cls.results['Q1']
         cls.res_drop = acf(cls.x, nlags=40, qstat=True, alpha=.05,
                            missing='drop', fft=False)
@@ -240,9 +245,11 @@ class TestACFMissing(CheckCorrGram):
                             DECIMAL_8)
 
     def test_qstat_none(self):
-        #todo why is res1/qstat 1 short
+        # todo why is res1/qstat 1 short
         assert_almost_equal(self.res_none[2], self.qstat_none, DECIMAL_3)
 
+
+# FIXME: enable/xfail/skip or delete
 # how to do this test? the correct q_stat depends on whether nobs=len(x) is
 # used when x contains NaNs or whether nobs<len(x) when x contains NaNs
 #    def test_qstat_drop(self):
@@ -258,7 +265,7 @@ class TestPACF(CheckCorrGram):
     def test_ols(self):
         pacfols, confint = pacf(self.x, nlags=40, alpha=.05, method="ols")
         assert_almost_equal(pacfols[1:], self.pacfols, DECIMAL_6)
-        centered = confint - confint.mean(1)[:,None]
+        centered = confint - confint.mean(1)[:, None]
         # from edited Stata ado file
         res = [[-.1375625, .1375625]] * 40
         assert_almost_equal(centered[1:41], res, DECIMAL_6)
@@ -292,7 +299,7 @@ class TestPACF(CheckCorrGram):
         assert_almost_equal(pacfyw, pacfld, DECIMAL_8)
 
         pacfyw = pacf(self.x, nlags=40, method="yw")
-        pacfld = pacf(self.x, nlags=40, method="ldu")
+        pacfld = pacf(self.x, nlags=40, method="lda")
         assert_almost_equal(pacfyw, pacfld, DECIMAL_8)
 
 
@@ -308,17 +315,18 @@ class CheckCoint(object):
     y2 = data.data['realgdp'].values
 
     def test_tstat(self):
-        assert_almost_equal(self.coint_t,self.teststat, DECIMAL_4)
+        assert_almost_equal(self.coint_t, self.teststat, DECIMAL_4)
 
 
-# this doesn't produce the old results anymore
+# this does not produce the old results anymore
 class TestCoint_t(CheckCoint):
     """
     Get AR(1) parameter on residuals
     """
+
     @classmethod
     def setup_class(cls):
-        #cls.coint_t = coint(cls.y1, cls.y2, trend="c")[0]
+        # cls.coint_t = coint(cls.y1, cls.y2, trend="c")[0]
         cls.coint_t = coint(cls.y1, cls.y2, trend="c", maxlag=0, autolag=None)[0]
         cls.teststat = -1.8208817
         cls.teststat = -1.830170986148
@@ -335,7 +343,8 @@ def test_coint():
     y += const
     y = np.round(y, 4)
 
-    for trend in []:#['c', 'ct', 'ctt', 'nc']:
+    # FIXME: enable/xfail/skip or delete
+    for trend in []:  # ['c', 'ct', 'ctt', 'nc']:
         print('\n', trend)
         print(coint(y[:, 0], y[:, 1], trend=trend, maxlag=4, autolag=None))
         print(coint(y[:, 0], y[:, 1:3], trend=trend, maxlag=4, autolag=None))
@@ -346,37 +355,37 @@ def test_coint():
     res_egranger = {}
     # trend = 'ct'
     res = res_egranger['ct'] = {}
-    res[0]  = [-5.615251442239, -4.406102369132,  -3.82866685109, -3.532082997903]
-    res[1]  = [-5.63591313706, -4.758609717199, -4.179130554708, -3.880909696863]
-    res[2]  = [-2.892029275027, -4.758609717199, -4.179130554708, -3.880909696863]
-    res[3]  = [-5.626932544079,  -5.08363327039, -4.502469783057,   -4.2031051091]
+    res[0] = [-5.615251442239, -4.406102369132, -3.82866685109, -3.532082997903]
+    res[1] = [-5.63591313706, -4.758609717199, -4.179130554708, -3.880909696863]
+    res[2] = [-2.892029275027, -4.758609717199, -4.179130554708, -3.880909696863]
+    res[3] = [-5.626932544079, -5.08363327039, -4.502469783057, -4.2031051091]
 
     # trend = 'c'
     res = res_egranger['c'] = {}
     # first critical value res[0][1] has a discrepancy starting at 4th decimal
-    res[0]  = [-5.760696844656, -3.952043522638, -3.367006313729, -3.065831247948]
+    res[0] = [-5.760696844656, -3.952043522638, -3.367006313729, -3.065831247948]
     # manually adjusted to have higher precision as in other cases
     res[0][1] = -3.952321293401682
-    res[1]  = [-5.781087068772, -4.367111915942, -3.783961136005, -3.483501524709]
-    res[2]  = [-2.477444137366, -4.367111915942, -3.783961136005, -3.483501524709]
-    res[3]  = [-5.778205811661, -4.735249216434, -4.152738973763, -3.852480848968]
+    res[1] = [-5.781087068772, -4.367111915942, -3.783961136005, -3.483501524709]
+    res[2] = [-2.477444137366, -4.367111915942, -3.783961136005, -3.483501524709]
+    res[3] = [-5.778205811661, -4.735249216434, -4.152738973763, -3.852480848968]
 
     # trend = 'ctt'
     res = res_egranger['ctt'] = {}
-    res[0]  = [-5.644431269946, -4.796038299708, -4.221469431008, -3.926472577178]
-    res[1]  = [-5.665691609506, -5.111158174219,  -4.53317278104,  -4.23601008516]
-    res[2]  = [-3.161462374828, -5.111158174219,  -4.53317278104,  -4.23601008516]
-    res[3]  = [-5.657904558563, -5.406880189412, -4.826111619543, -4.527090164875]
+    res[0] = [-5.644431269946, -4.796038299708, -4.221469431008, -3.926472577178]
+    res[1] = [-5.665691609506, -5.111158174219, -4.53317278104, -4.23601008516]
+    res[2] = [-3.161462374828, -5.111158174219, -4.53317278104, -4.23601008516]
+    res[3] = [-5.657904558563, -5.406880189412, -4.826111619543, -4.527090164875]
 
     # The following for 'nc' are only regression test numbers
     # trend = 'nc' not allowed in egranger
     # trend = 'nc'
     res = res_egranger['nc'] = {}
     nan = np.nan  # shortcut for table
-    res[0]  = [-3.7146175989071137, nan, nan, nan]
-    res[1]  = [-3.8199323012888384, nan, nan, nan]
-    res[2]  = [-1.6865000791270679, nan, nan, nan]
-    res[3]  = [-3.7991270451873675, nan, nan, nan]
+    res[0] = [-3.7146175989071137, nan, nan, nan]
+    res[1] = [-3.8199323012888384, nan, nan, nan]
+    res[2] = [-1.6865000791270679, nan, nan, nan]
+    res[3] = [-3.7991270451873675, nan, nan, nan]
 
     for trend in ['c', 'ct', 'ctt', 'nc']:
         res1 = {}
@@ -437,26 +446,50 @@ class TestGrangerCausality(object):
         data = mdata.astype(float)
         data = np.diff(np.log(data), axis=0)
 
-        #R: lmtest:grangertest
+        # R: lmtest:grangertest
         r_result = [0.243097, 0.7844328, 195, 2]  # f_test
         gr = grangercausalitytests(data[:, 1::-1], 2, verbose=False)
         assert_almost_equal(r_result, gr[2][0]['ssr_ftest'], decimal=7)
         assert_almost_equal(gr[2][0]['params_ftest'], gr[2][0]['ssr_ftest'],
                             decimal=7)
 
+    def test_grangercausality_single(self):
+        mdata = macrodata.load_pandas().data
+        mdata = mdata[['realgdp', 'realcons']].values
+        data = mdata.astype(float)
+        data = np.diff(np.log(data), axis=0)
+        gr = grangercausalitytests(data[:, 1::-1], 2, verbose=False)
+        gr2 = grangercausalitytests(data[:, 1::-1], [2], verbose=False)
+        assert 1 in gr
+        assert 1 not in gr2
+        assert_almost_equal(gr[2][0]['ssr_ftest'], gr2[2][0]['ssr_ftest'],
+                            decimal=7)
+        assert_almost_equal(gr[2][0]['params_ftest'], gr2[2][0]['ssr_ftest'],
+                            decimal=7)
+
     def test_granger_fails_on_nobs_check(self, reset_randomstate):
         # Test that if maxlag is too large, Granger Test raises a clear error.
-        X = np.random.rand(10, 2)
-        grangercausalitytests(X, 2, verbose=False)  # This should pass.
-        assert_raises(ValueError, grangercausalitytests, X, 3, verbose=False)
+        x = np.random.rand(10, 2)
+        grangercausalitytests(x, 2, verbose=False)  # This should pass.
+        with pytest.raises(ValueError):
+            grangercausalitytests(x, 3, verbose=False)
+
+    def test_granger_fails_on_finite_check(self, reset_randomstate):
+        x = np.random.rand(1000, 2)
+        x[500, 0] = np.nan
+        x[750, 1] = np.inf
+        with pytest.raises(ValueError, match="x contains NaN"):
+            grangercausalitytests(x, 2)
+
+    def test_granger_fails_on_zero_lag(self, reset_randomstate):
+        x = np.random.rand(1000, 2)
+        with pytest.raises(
+                ValueError,
+                match="maxlag must be a non-empty list containing only positive integers"):
+            grangercausalitytests(x, [0, 1, 2])
 
 
-class SetupKPSS(object):
-    data = macrodata.load_pandas()
-    x = data.data['realgdp'].values
-
-
-class TestKPSS(SetupKPSS):
+class TestKPSS:
     """
     R-code
     ------
@@ -467,11 +500,15 @@ class TestKPSS(SetupKPSS):
     In this context, x is the vector containing the
     macrodata['realgdp'] series.
     """
+    @classmethod
+    def setup(cls):
+        cls.data = macrodata.load_pandas()
+        cls.x = cls.data.data['realgdp'].values
 
     def test_fail_nonvector_input(self, reset_randomstate):
         # should be fine
         with pytest.warns(InterpolationWarning):
-            kpss(self.x, lags='legacy')
+            kpss(self.x, nlags='legacy')
 
         x = np.random.rand(20, 2)
         assert_raises(ValueError, kpss, x)
@@ -479,16 +516,16 @@ class TestKPSS(SetupKPSS):
     def test_fail_unclear_hypothesis(self):
         # these should be fine,
         with pytest.warns(InterpolationWarning):
-            kpss(self.x, 'c', lags='legacy')
+            kpss(self.x, 'c', nlags='legacy')
         with pytest.warns(InterpolationWarning):
-            kpss(self.x, 'C', lags='legacy')
+            kpss(self.x, 'C', nlags='legacy')
         with pytest.warns(InterpolationWarning):
-            kpss(self.x, 'ct', lags='legacy')
+            kpss(self.x, 'ct', nlags='legacy')
         with pytest.warns(InterpolationWarning):
-            kpss(self.x, 'CT', lags='legacy')
+            kpss(self.x, 'CT', nlags='legacy')
 
         assert_raises(ValueError, kpss, self.x, "unclear hypothesis",
-                      lags='legacy')
+                      nlags='legacy')
 
     def test_teststat(self):
         with pytest.warns(InterpolationWarning):
@@ -520,22 +557,22 @@ class TestKPSS(SetupKPSS):
     def test_lags(self):
         # real GDP from macrodata data set
         with pytest.warns(InterpolationWarning):
-            res = kpss(self.x, 'c', lags='auto')
+            res = kpss(self.x, 'c', nlags='auto')
         assert_equal(res[2], 9)
         # real interest rates from macrodata data set
-        res = kpss(sunspots.load(True).data['SUNACTIVITY'], 'c', lags='auto')
+        res = kpss(sunspots.load(True).data['SUNACTIVITY'], 'c', nlags='auto')
         assert_equal(res[2], 7)
         # volumes from nile data set
         with pytest.warns(InterpolationWarning):
-            res = kpss(nile.load(True).data['volume'], 'c', lags='auto')
+            res = kpss(nile.load(True).data['volume'], 'c', nlags='auto')
         assert_equal(res[2], 5)
         # log-coinsurance from randhie data set
         with pytest.warns(InterpolationWarning):
-            res = kpss(randhie.load(True).data['lncoins'], 'ct', lags='auto')
+            res = kpss(randhie.load(True).data['lncoins'], 'ct', nlags='auto')
         assert_equal(res[2], 75)
         # in-vehicle time from modechoice data set
         with pytest.warns(InterpolationWarning):
-            res = kpss(modechoice.load(True).data['invt'], 'ct', lags='auto')
+            res = kpss(modechoice.load(True).data['invt'], 'ct', nlags='auto')
         assert_equal(res[2], 18)
 
     def test_kpss_fails_on_nobs_check(self):
@@ -546,10 +583,11 @@ class TestKPSS(SetupKPSS):
         msg = (r"lags \({}\) must be < number of observations \({}\)"
                .format(nobs, nobs))
         with pytest.raises(ValueError, match=msg):
-            kpss(self.x, 'c', lags=nobs)
+            kpss(self.x, 'c', nlags=nobs)
 
     def test_kpss_autolags_does_not_assign_lags_equal_to_nobs(self):
-        # Test that if *autolags* exceeds number of observations, we set suitable lags
+        # Test that if *autolags* exceeds number of observations, we set
+        # suitable lags
         # GH5925
         data_which_breaks_autolag = np.array(
             [0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0,
@@ -567,18 +605,18 @@ class TestKPSS(SetupKPSS):
              1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1,
              1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0]).astype(float)
 
-        kpss(data_which_breaks_autolag, lags="auto")
+        kpss(data_which_breaks_autolag, nlags="auto")
 
     def test_legacy_lags(self):
         # Test legacy lags are the same
         with pytest.warns(InterpolationWarning):
-            res = kpss(self.x, 'c', lags='legacy')
+            res = kpss(self.x, 'c', nlags='legacy')
         assert_equal(res[2], 15)
 
     def test_unknown_lags(self):
         # Test legacy lags are the same
         with pytest.raises(ValueError):
-            kpss(self.x, 'c', lags='unknown')
+            kpss(self.x, 'c', nlags='unknown')
 
     def test_deprecation(self):
         with pytest.warns(FutureWarning):
@@ -602,13 +640,13 @@ def test_acovf2d(reset_randomstate):
 
 
 @pytest.mark.parametrize('demean', [True, False])
-@pytest.mark.parametrize('unbiased', [True, False])
-def test_acovf_fft_vs_convolution(demean, unbiased):
+@pytest.mark.parametrize('adjusted', [True, False])
+def test_acovf_fft_vs_convolution(demean, adjusted):
     np.random.seed(1)
     q = np.random.normal(size=100)
 
-    F1 = acovf(q, demean=demean, unbiased=unbiased, fft=True)
-    F2 = acovf(q, demean=demean, unbiased=unbiased, fft=False)
+    F1 = acovf(q, demean=demean, adjusted=adjusted, fft=True)
+    F2 = acovf(q, demean=demean, adjusted=adjusted, fft=False)
     assert_almost_equal(F1, F2, decimal=7)
 
 
@@ -627,16 +665,16 @@ def test_arma_order_select_ic():
     y = arma_generate_sample(arparams, maparams, nobs)
     res = arma_order_select_ic(y, ic=['aic', 'bic'], trend='nc')
     # regression tests in case we change algorithm to minic in sas
-    aic_x = np.array([[       np.nan,  552.7342255 ,  484.29687843],
-                      [ 562.10924262,  485.5197969 ,  480.32858497],
-                      [ 507.04581344,  482.91065829,  481.91926034],
-                      [ 484.03995962,  482.14868032,  483.86378955],
-                      [ 481.8849479 ,  483.8377379 ,  485.83756612]])
-    bic_x = np.array([[       np.nan,  559.77714733,  494.86126118],
-                      [ 569.15216446,  496.08417966,  494.41442864],
-                      [ 517.61019619,  496.99650196,  499.52656493],
-                      [ 498.12580329,  499.75598491,  504.99255506],
-                      [ 499.49225249,  504.96650341,  510.48779255]])
+    aic_x = np.array([[np.nan, 552.7342255, 484.29687843],
+                      [562.10924262, 485.5197969, 480.32858497],
+                      [507.04581344, 482.91065829, 481.91926034],
+                      [484.03995962, 482.14868032, 483.86378955],
+                      [481.8849479, 483.8377379, 485.83756612]])
+    bic_x = np.array([[np.nan, 559.77714733, 494.86126118],
+                      [569.15216446, 496.08417966, 494.41442864],
+                      [517.61019619, 496.99650196, 499.52656493],
+                      [498.12580329, 499.75598491, 504.99255506],
+                      [499.49225249, 504.96650341, 510.48779255]])
     aic = DataFrame(aic_x, index=lrange(5), columns=lrange(3))
     bic = DataFrame(bic_x, index=lrange(5), columns=lrange(3))
     assert_almost_equal(res.aic.values, aic.values, 5)
@@ -668,16 +706,16 @@ def test_arma_order_select_ic_failure():
     # this should trigger an SVD convergence failure, smoke test that it
     # returns, likely platform dependent failure...
     # looks like AR roots may be cancelling out for 4, 1?
-    y = np.array([ 0.86074377817203640006,  0.85316549067906921611,
-        0.87104653774363305363,  0.60692382068987393851,
-        0.69225941967301307667,  0.73336177248909339976,
-        0.03661329261479619179,  0.15693067239962379955,
-        0.12777403512447857437, -0.27531446294481976   ,
-       -0.24198139631653581283, -0.23903317951236391359,
-       -0.26000241325906497947, -0.21282920015519238288,
-       -0.15943768324388354896,  0.25169301564268781179,
-        0.1762305709151877342 ,  0.12678133368791388857,
-        0.89755829086753169399,  0.82667068795350151511])
+    y = np.array([0.86074377817203640006, 0.85316549067906921611,
+                  0.87104653774363305363, 0.60692382068987393851,
+                  0.69225941967301307667, 0.73336177248909339976,
+                  0.03661329261479619179, 0.15693067239962379955,
+                  0.12777403512447857437, -0.27531446294481976,
+                  -0.24198139631653581283, -0.23903317951236391359,
+                  -0.26000241325906497947, -0.21282920015519238288,
+                  -0.15943768324388354896, 0.25169301564268781179,
+                  0.1762305709151877342, 0.12678133368791388857,
+                  0.89755829086753169399, 0.82667068795350151511])
     import warnings
     with warnings.catch_warnings():
         # catch a hessian inversion and convergence failure warning
@@ -688,29 +726,28 @@ def test_arma_order_select_ic_failure():
 def test_acf_fft_dataframe():
     # regression test #322
 
-    result = acf(sunspots.load_pandas().data[['SUNACTIVITY']], fft=True)
+    result = acf(sunspots.load_pandas().data[['SUNACTIVITY']], fft=True, nlags=20)
     assert_equal(result.ndim, 1)
 
 
 def test_levinson_durbin_acov():
     rho = 0.9
     m = 20
-    acov = rho**np.arange(200)
+    acov = rho ** np.arange(200)
     sigma2_eps, ar, pacf, _, _ = levinson_durbin(acov, m, isacov=True)
     assert_allclose(sigma2_eps, 1 - rho ** 2)
     assert_allclose(ar, np.array([rho] + [0] * (m - 1)), atol=1e-8)
     assert_allclose(pacf, np.array([1, rho] + [0] * (m - 1)), atol=1e-8)
 
 
-
 @pytest.mark.parametrize("missing", ['conservative', 'drop', 'raise', 'none'])
 @pytest.mark.parametrize("fft", [False, True])
 @pytest.mark.parametrize("demean", [True, False])
-@pytest.mark.parametrize("unbiased", [True, False])
-def test_acovf_nlags(acovf_data, unbiased, demean, fft, missing):
-    full = acovf(acovf_data, unbiased=unbiased, demean=demean, fft=fft,
+@pytest.mark.parametrize("adjusted", [True, False])
+def test_acovf_nlags(acovf_data, adjusted, demean, fft, missing):
+    full = acovf(acovf_data, adjusted=adjusted, demean=demean, fft=fft,
                  missing=missing)
-    limited = acovf(acovf_data, unbiased=unbiased, demean=demean, fft=fft,
+    limited = acovf(acovf_data, adjusted=adjusted, demean=demean, fft=fft,
                     missing=missing, nlag=10)
     assert_allclose(full[:11], limited)
 
@@ -718,13 +755,13 @@ def test_acovf_nlags(acovf_data, unbiased, demean, fft, missing):
 @pytest.mark.parametrize("missing", ['conservative', 'drop'])
 @pytest.mark.parametrize("fft", [False, True])
 @pytest.mark.parametrize("demean", [True, False])
-@pytest.mark.parametrize("unbiased", [True, False])
-def test_acovf_nlags_missing(acovf_data, unbiased, demean, fft, missing):
+@pytest.mark.parametrize("adjusted", [True, False])
+def test_acovf_nlags_missing(acovf_data, adjusted, demean, fft, missing):
     acovf_data = acovf_data.copy()
     acovf_data[1:3] = np.nan
-    full = acovf(acovf_data, unbiased=unbiased, demean=demean, fft=fft,
+    full = acovf(acovf_data, adjusted=adjusted, demean=demean, fft=fft,
                  missing=missing)
-    limited = acovf(acovf_data, unbiased=unbiased, demean=demean, fft=fft,
+    limited = acovf(acovf_data, adjusted=adjusted, demean=demean, fft=fft,
                     missing=missing, nlag=10)
     assert_allclose(full[:11], limited)
 
@@ -777,7 +814,7 @@ def test_pacf2acf_errors():
     with pytest.raises(ValueError):
         levinson_durbin_pacf(pacf, nlags=20)
     with pytest.raises(ValueError):
-        levinson_durbin_pacf(pacf[:1])
+        levinson_durbin_pacf(pacf[1:])
     with pytest.raises(ValueError):
         levinson_durbin_pacf(np.zeros(10))
     with pytest.raises(ValueError):
@@ -801,7 +838,7 @@ def test_pacf_burg():
 
 def test_pacf_burg_error():
     with pytest.raises(ValueError):
-        pacf_burg(np.empty((20,2)), 10)
+        pacf_burg(np.empty((20, 2)), 10)
     with pytest.raises(ValueError):
         pacf_burg(np.empty(100), 101)
 
@@ -831,13 +868,13 @@ def test_innovations_algo_rtol():
 def test_innovations_errors():
     ma = -0.9
     acovf = np.array([1 + ma ** 2, ma])
-    with pytest.raises(ValueError):
+    with pytest.raises(TypeError):
         innovations_algo(acovf, nobs=2.2)
     with pytest.raises(ValueError):
         innovations_algo(acovf, nobs=-1)
     with pytest.raises(ValueError):
         innovations_algo(np.empty((2, 2)))
-    with pytest.raises(ValueError):
+    with pytest.raises(TypeError):
         innovations_algo(acovf, rtol='none')
 
 
@@ -898,7 +935,7 @@ def test_innovations_algo_filter_kalman_filter(reset_randomstate):
 
     theta, v = innovations_algo(acovf)
     u = innovations_filter(endog, theta)
-    llf_obs = -0.5 * u**2 / (sigma2 * v) - 0.5 * np.log(2 * np.pi * v)
+    llf_obs = -0.5 * u ** 2 / (sigma2 * v) - 0.5 * np.log(2 * np.pi * v)
 
     # Kalman filter apparoach
     mod = SARIMAX(endog, order=(len(ar_params), 0, len(ma_params)))
@@ -906,7 +943,7 @@ def test_innovations_algo_filter_kalman_filter(reset_randomstate):
 
     # Test that the two approaches are identical
     atol = 1e-6 if PLATFORM_WIN else 0.0
-    assert_allclose(u, res.forecasts_error[0], atol=atol)
+    assert_allclose(u, res.forecasts_error[0], rtol=1e-6, atol=atol)
     assert_allclose(theta[1:, 0], res.filter_results.kalman_gain[0, 0, :-1],
                     atol=atol)
     assert_allclose(llf_obs, res.llf_obs, atol=atol)
@@ -928,3 +965,94 @@ def test_adfuller_maxlag_too_large(reset_randomstate):
     y = np.random.standard_normal(100)
     with pytest.raises(ValueError, match='maxlag must be less than'):
         adfuller(y, maxlag=51)
+
+
+class SetupZivotAndrews(object):
+    # test directory
+    cur_dir = CURR_DIR
+    run_dir = os.path.join(cur_dir, 'results')
+    # use same file for testing failure modes
+    fail_file = os.path.join(run_dir, 'rgnp.csv')
+    fail_mdl = np.asarray(pd.read_csv(fail_file))
+
+
+class TestZivotAndrews(SetupZivotAndrews):
+
+    # failure mode tests
+    def test_fail_regression_type(self):
+        with pytest.raises(ValueError):
+            zivot_andrews(self.fail_mdl, regression='x')
+
+    def test_fail_trim_value(self):
+        with pytest.raises(ValueError):
+            zivot_andrews(self.fail_mdl, trim=0.5)
+
+    def test_fail_array_shape(self):
+        with pytest.raises(ValueError):
+            zivot_andrews(np.random.rand(50, 2))
+
+    def test_fail_autolag_type(self):
+        with pytest.raises(ValueError):
+            zivot_andrews(self.fail_mdl, autolag='None')
+
+    @pytest.mark.parametrize('autolag', ["AIC", "aic", "Aic"])
+    def test_autolag_case_sensitivity(self, autolag):
+        res = zivot_andrews(self.fail_mdl, autolag=autolag)
+        assert res[3] == 1
+
+    # following tests compare results to R package urca.ur.za (1.13-0)
+    def test_rgnp_case(self):
+        res = zivot_andrews(self.fail_mdl, maxlag=8, regression='c',
+                            autolag=None)
+        assert_allclose([res[0], res[1], res[4]],
+                        [-5.57615, 0.00312, 20], rtol=1e-3)
+
+    def test_gnpdef_case(self):
+        mdlfile = os.path.join(self.run_dir, 'gnpdef.csv')
+        mdl = np.asarray(pd.read_csv(mdlfile))
+        res = zivot_andrews(mdl, maxlag=8, regression='c', autolag='t-stat')
+        assert_allclose([res[0], res[1], res[3], res[4]],
+                        [-4.12155, 0.28024, 5, 40], rtol=1e-3)
+
+    def test_stkprc_case(self):
+        mdlfile = os.path.join(self.run_dir, 'stkprc.csv')
+        mdl = np.asarray(pd.read_csv(mdlfile))
+        res = zivot_andrews(mdl, maxlag=8, regression='ct', autolag='t-stat')
+        assert_allclose([res[0], res[1], res[3], res[4]],
+                        [-5.60689, 0.00894, 1, 65], rtol=1e-3)
+
+    def test_rgnpq_case(self):
+        mdlfile = os.path.join(self.run_dir, 'rgnpq.csv')
+        mdl = np.asarray(pd.read_csv(mdlfile))
+        res = zivot_andrews(mdl, maxlag=12, regression='t', autolag='t-stat')
+        assert_allclose([res[0], res[1], res[3], res[4]],
+                        [-3.02761, 0.63993, 12, 102], rtol=1e-3)
+
+    def test_rand10000_case(self):
+        mdlfile = os.path.join(self.run_dir, 'rand10000.csv')
+        mdl = np.asarray(pd.read_csv(mdlfile))
+        res = zivot_andrews(mdl, regression='c', autolag='t-stat')
+        assert_allclose([res[0], res[1], res[3], res[4]],
+                        [-3.48223, 0.69111, 25, 7071], rtol=1e-3)
+
+
+def test_acf_conservate_nanops(reset_randomstate):
+    # GH 6729
+    e = np.random.standard_normal(100)
+    for i in range(1, e.shape[0]):
+        e[i] += 0.9 * e[i-1]
+    e[::7] = np.nan
+    result = acf(e, missing="conservative", nlags=10, fft=False)
+    resid = e - np.nanmean(e)
+    expected = np.ones(11)
+    nobs = e.shape[0]
+    gamma0 = np.nansum(resid * resid)
+    for i in range(1, 10+1):
+        expected[i] = np.nansum(resid[i:] * resid[:nobs-i]) / gamma0
+    assert_allclose(result, expected, rtol=1e-4, atol=1e-4)
+
+
+def test_pacf_nlags_error(reset_randomstate):
+    e = np.random.standard_normal(100)
+    with pytest.raises(ValueError, match="Can only compute partial"):
+        pacf(e, 50)
